/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86i                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for Fujitsu F2MC-F16LX 90550A series)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          You may modify this file acc. to your target system, but 
          modification has to be trcked, if updates are used later.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Check Configuration
*
**********************************************************************
*/

#include "_f16lxs.h"                /* Examine and check cpu series */

#ifndef __CPU_MB90550A_SERIES
  #error "This RTOSInit.c is for MB90550A series. Please check project settings"
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#include "mb90550.h"

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS
  /*  Internal CPU clock. Value depends on external crystal         */
  /*  and PLL settings.                                             */
  #define OS_FSYS 16000000L
#endif
#ifndef OS_PCLK_TIMER
  /*  Internal peripheral clock for embOS timer. Value depends on   */
  /*  CPU clock and selected prescaler for timer clock              */
  #define OS_PCLK_TIMER (OS_FSYS/2)
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART0
*/

#ifndef   OS_UART
  #define OS_UART 0
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _Timer_IRQHandler
*/
static void _Timer_IRQHandler (void) {
  TMCSR0 &= ~(1<<2);                 /* Reset Timer UF (clear Int)  */
  OS_HandleTick();                   /* embOS tick handler          */
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize system timer with 1ms resolution
*       Reload timer 0 is used. If any other timer should be used,
*       modify this init routine and also modify interrupt vector
*/
void OS_InitHW(void) {
  ICR07  = 7;                      /* First disable interrupt       */
  TMRLR0 = (OS_PCLK_TIMER/1000-1); /* Generate Interrupts every 1ms */
  TMCSR0 = (0<<10)                 /* Clock source: uPClk div. 2    */
          |(1<<4)                  /* Reload Mode                   */
          |(1<<3)                  /* Interrupt enable              */
          |(1<<1)                  /* Count enable                  */
          |(1<<0);                 /* Trigger to start timer        */
  ICR07  = 6;                      /* Enable, lowest priority       */
  OS_COM_Init();                   /* Initialize comm. to embOSView */
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  OS_EI();
  for (;;);
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via embOSView or high 
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/

OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time = OS_Time+1;
  unsigned int t_cnt = TMR0;
  if (TMCSR0_UF) {   /* If timer interrupt pending, */
    t_cnt = TMR0;    /* correct result              */
    time++;
  }
  return (OS_PCLK_TIMER/1000)*time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_Timer_IRQHandler
*/
__interrupt void OS_Timer_IRQHandler (void) {
  OS_CallNestableISR(_Timer_IRQHandler);      /* Call the timer handler which runs on user stack */
}

/*********************************************************************
*
*       Communication for embOSView (UART 0)
*
**********************************************************************
*/

#if OS_UART == 0

#if   ((OS_BAUDRATE == 38400) && (OS_FSYS == 16000000))
  #define BRG_DIV 0x0E
#elif ((OS_BAUDRATE == 19200) && (OS_FSYS == 16000000))
  #define BRG_DIV 0x0C
#elif ((OS_BAUDRATE == 9600)  && (OS_FSYS == 16000000))
  #define BRG_DIV 0x08
#else
  #error "Settings OS_BAUDRATE or OS_FSYS not supported, please check settings!"
#endif

void OS_COM_Init(void) {
  SCR     = (0<<7)       /*  Disable Parity                      */
           |(0<<6)       /*  Parity EVEN                         */
           |(0<<5)       /*  1 Stop bit                          */
           |(1<<4)       /*  8 Data bits                         */
           |(0<<3)       /*  Data Frame                          */
           |(0<<3)       /*  Reset Rx error                      */
           |(0<<1)       /*  Disable Rx                          */
           |(0<<0);      /*  Disable Tx                          */
  SSR      = 0x00;       /*  Disable Rx- Tx- interrupts          */
  SMR      = 0x01;       /*  Set normal async mode, SOT enabled  */
  CDCR     = (1<<7)      /*  Start communication prescaler       */
           | BRG_DIV;    /*  at selected baudrate baudrate       */
  ICR13    = 0x06;       /*  Setup ICR Tx, lowest priority       */
  ICR14    = 0x06;       /*  Setup ICR Rx, lowest priority       */
  SCR_RXE  = 1;          /*  Enable Rx, Tx                       */
  SCR_TXE  = 1;
  SSR_RIE  = 1;          /*  Enable Interrupts                   */
  SSR_TIE  = 1;
}

/*********************************************************************
*
*       _ISR_tx()
*       Tx interrupt handler for UART1
*/
static void _ISR_tx (void) {
  if (OS_OnTx()) { /*  No more characters to send ? */
    SSR_TIE = 0;   /*  => disable Tx interrupt      */
  }
}

/*********************************************************************
*
*       _ISR_rx()
*       Rx interrupt handler for UART1
*/
static void OS_Com_ISR_rx (void) {
  int Data;

  Data = SIDR;
  if (SSR & 0xE0) {   /* Check if errors occurred */
    SCR_REC = 0;      /* Reset error              */
  } else {
    OS_OnRx(Data);
  }
}

/*********************************************************************
*
*       OS_COM_Send1() for UART1:
*       Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(unsigned char c) {
  SODR = c;            /* Write character     */
  SSR_TIE = 1;         /* Enable Tx interrupt */
}

/*********************************************************************
*
*       OS_Com_ISR_tx
*       Tx interrupt handler for UART1
*/
__interrupt void OS_Com_ISR_tx (void) {
  OS_CallNestableISR(_ISR_rx);
}

/*********************************************************************
*
*       OS_Com_ISR_rx
*       Rx interrupt handler for UART1
*/
__interrupt void OS_Com_ISR_rx (void) {
  OS_CallNestableISR(_ISR_rx);
}

#else  /* UART handlers not defined, using dummies */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);           /* Avoid compiler warning                   */
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif /* (OS_UART==0)

/*********************************************************************
*
*       Interrupt vector table
*
**********************************************************************
*/

/*********************************************************************
*
*       embOS timer interrupt vector
*       Has to be modified, if other timer is used
*/

#pragma intvect OS_Timer_IRQHandler 25   /* 16-bit reload timer #0  */

/*********************************************************************
*
*       embOS UART interrupt vectors for communication to embOSView
*/

#if OS_UART == 0
  #pragma  intvect OS_Com_ISR_tx 37
  #pragma  intvect OS_Com_ISR_rx 39
#endif

/*********************************************************************
*
*       SEGGER internal test interrupt. Used to test embOS on
*       hardware.
*       You may remove this section
*/

#ifdef OSTEST
  extern __interrupt void OSTEST_X_ISR0(void);
  #pragma  intvect OSTEST_X_ISR0 27        /* Reload timer #1 used */
#endif
  
/*****  End Of File  ************************************************/

