/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86i                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for Fujitsu F2MC-F16LX 90590 series)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          You may modify this file acc. to your target system, but 
          modification has to be trcked, if updates are used later.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Check Configuration
*
**********************************************************************
*/

#include "_f16lxs.h"                /* Examine and check cpu series */

#ifndef __CPU_MB90590_SERIES
  #error "This RTOSInit.c is for MB90590 series. Please check project settings"
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#include "mb90590.h"

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS
  /*  Internal CPU clock. Value depends on external crystal         */
  /*  and PLL settings.                                             */
  #define OS_FSYS 16000000L
#endif
#ifndef OS_PCLK_TIMER
  /*  Internal peripheral clock for embOS timer. Value depends on   */
  /*  CPU clock and selected prescaler for timer clock              */
  #define OS_PCLK_TIMER (OS_FSYS/2)
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 1 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 1
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _Timer_IRQHandler
*/
static void _Timer_IRQHandler (void) {
  TMCSR0 &= ~(1<<2);                 /* Reset Timer UF (clear Int)  */
  OS_HandleTick();                   /* embOS tick handler          */
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize system timer with 1ms resolution
*       Reload timer 0 is used. If any other timer should be used,
*       modify this init routine and also modify interrupt vector
*/
void OS_InitHW(void) {
  ICR06  = 7;                      /* First disable interrupt       */
  TMRLR0 = (OS_PCLK_TIMER/1000-1); /* Generate Interrupts every 1ms */
  TMCSR0 = (0<<10)                 /* Clock source: uPClk div. 2    */
          |(1<<4)                  /* Reload Mode                   */
          |(1<<3)                  /* Interrupt enable              */
          |(1<<1)                  /* Count enable                  */
          |(1<<0);                 /* Trigger to start timer        */
  ICR06  = 6;                      /* Enable, lowest priority       */
  OS_COM_Init();                   /* Initialize comm. to embOSView */
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  OS_EI();
  for (;;);
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via embOSView or high 
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time = OS_Time+1;
  unsigned int t_cnt = TMR0;
  if (TMCSR0_UF) {   /* If timer interrupt pending, */
    t_cnt = TMR0;    /* correct result              */
    time++;
  }
  return (OS_PCLK_TIMER/1000)*time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_Timer_IRQHandler
*/
__interrupt void OS_Timer_IRQHandler (void) {
  OS_CallNestableISR(_Timer_IRQHandler);      /* Call the timer handler which runs on user stack */
}

/*********************************************************************
*
*       Communication for embOSView (UART0 or UART1)
*
**********************************************************************
*/

#if (OS_UART == 0) || (OS_UART == 1)
  #if   ((OS_BAUDRATE == 38400) && (OS_FSYS == 16000000))
    #define BRG_DIV 0x0C
  #elif ((OS_BAUDRATE == 19200) && (OS_FSYS == 16000000))
    #define BRG_DIV 0x2C
  #elif ((OS_BAUDRATE == 9600)  && (OS_FSYS == 16000000))
    #define BRG_DIV 0x3C
  #else
    #error "Settings OS_BAUDRATE or OS_FSYS not supported, please check settings!"
  #endif

/********************************************************************
*
*       Configuration and Settings for UART0
*/

#if OS_UART == 0
  #define URD URD0
  #define UMC UMC0
  #define USR USR0
  #define ENABLE_SOT() (DDR3_D34 = 1)
  #define COM_ICR ICR12
  #define UODR UODR0
  #define UIDR UIDR0
  #define USR_TIE USR0_TIE
  #define UMC_RFC UMC0_RFC
  #define USR_ORFE USR0_ORFE

/********************************************************************
*
*       Configuration and Settings for UART1
*/

#elif OS_UART == 1
  #define URD URD1
  #define UMC UMC1
  #define USR USR1
  #define ENABLE_SOT()
  #define COM_ICR ICR13
  #define UODR UODR1
  #define UIDR UIDR1
  #define USR_TIE USR1_TIE
  #define UMC_RFC UMC1_RFC
  #define USR_ORFE USR1_ORFE
#endif

/*********************************************************************
*
*       _ISR_tx() 
*       Tx interrupt handler for UART
*/
static void _ISR_tx (void) {
  if (OS_OnTx()) {    /*  No more characters to send ? */
    USR_TIE = 0;      /*  => disable Tx interrupt      */
  }
}

/*********************************************************************
*
*       _ISR_rx()
*       Rx interrupt handler for UART
*/
static void _ISR_rx (void) {
  int Data;

  Data = UIDR;
  if (USR_ORFE) {   /* Check if errors occurred */
    UMC_RFC = 0;    /* Reset error              */
  } else {
    OS_OnRx(Data);
  }
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize selected COM port for embOSView
*/
void OS_COM_Init(void) {
  ENABLE_SOT();        /* Enable SOT as output             */
  USR      = 0;        /* Disable all interrupts           */
  COM_ICR  = 0x06;     /* Enable ICR, lowest priority      */
  URD      = BRG_DIV;  /* Setup Baudrate                   */
  UMC      = 0x19;     /* Set 8N1 format, async, SO enable */
  USR      = 0x0C;     /* Enable Rx, Tx interrupts         */
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(unsigned char c) {
  UODR = c;           /* Write character      */
  USR_TIE = 1;        /* Enable Tx interrupt  */
}

/*********************************************************************
*
*       OS_Com_ISR_tx() 
*       Tx interrupt handler for UART
*/
__interrupt void OS_Com_ISR_tx (void) {
  OS_CallNestableISR(_ISR_tx);
}

/*********************************************************************
*
*       OS_Com_ISR_rx()
*       Rx interrupt handler for UART
*/
__interrupt void OS_Com_ISR_rx (void) {
  OS_CallNestableISR(_ISR_rx);
}

#else  /* UART handlers not defined, using dummies */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);           /* Avoid compiler warning                   */
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif /* (OS_UART==0) || (OS_UART==1) */

/*********************************************************************
*
*       Interrupt vector table
*
**********************************************************************
*/

/*********************************************************************
*
*       embOS timer interrupt vector
*       Has to be modified, if other timer is used
*/

#pragma intvect OS_Timer_IRQHandler 23   /* 16-bit reload timer #0  */

/*********************************************************************
*
*       embOS UART interrupt vectors for communication to embOSView
*/

#if OS_UART == 0
  #pragma intvect OS_Com_ISR_rx 35
  #pragma intvect OS_Com_ISR_tx 36
#elif OS_UART == 1
  #pragma intvect OS_Com_ISR_rx 37
  #pragma intvect OS_Com_ISR_tx 38
#endif

/*********************************************************************
*
*       SEGGER internal test interrupt. Used to test embOS on
*       hardware.
*       You may remove this section
*/

#ifdef OSTEST 
  extern __interrupt void OSTEST_X_ISR0(void);
  #pragma intvect OSTEST_X_ISR0  26     /* I/O timer compare 0 used */
#endif

/*****  End Of File  ************************************************/
