/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86i                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_MB96F348.c   (for Fujitsu F2MC-F16FX 96340 series)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          You may modify this file acc. to your target system, but 
          modification has to be trcked, if updates are used later.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "mb96348ts.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

/*  Internal CPU clock. Value depends on external crystal         */
/*  and PLL settings.                                             */
#ifndef   OS_FSYS
  #define OS_FSYS 56000000L
#endif

/*  Internal peripheral clock for embOS timer. Value depends on   */
/*  CPU clock and selected prescaler for timer clock              */

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER (OS_FSYS/4)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS)
#endif

#define _OS_TIMER_INT_INDEX    51
#define _OS_TIMER_INT_PRIORITY 6

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 1 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 1
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define _OS_UART_RX_INT_PRIORITY 6   // Rx interrupt priority, lowest
#define _OS_UART_TX_INT_PRIORITY 6   // Tx interrupt priority, lowest

/****** End of configuration settings *******************************/

#define OS_UART_USED ((OS_UART == 0) || (OS_UART == 1))

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _ISR_Tick()
*/
static void _ISR_Tick (void) {
  TMCSR0 &= ~(1 << 2);           /* Reset Timer UF (clear Int)      */
  OS_HandleTick();               /* Call the embOS tick handler     */
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize system timer with 1ms resolution
*       Reload timer 0 is used. If any other timer should be used,
*       modify this init routine and also modify interrupt vector
*/
void OS_InitHW(void) {
  OS_IncDI();                      // Disable embOS interrupts  
  /* Disable timer interrupt */
  ICR = (_OS_TIMER_INT_INDEX << 8) | 7;
  TMRLR0 = (OS_PCLK_TIMER/1000-1); /* Generate Interrupts every 1ms */
  TMCSR0 = (0 << 10)               /* Clock source: CLKP1 / 4       */
          |(1 << 4)                /* Reload Mode                   */
          |(1 << 3)                /* Interrupt enable              */
          |(1 << 1)                /* Count enable                  */
          |(1 << 0);               /* Trigger to start timer        */
  // Enable timer interrupt
  ICR = (_OS_TIMER_INT_INDEX << 8) | _OS_TIMER_INT_PRIORITY;
  OS_COM_Init();                   /* Initialize comm. to embOSView */
  OS_DecRI();                      // Restore embOS interrupt state
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  OS_EI();
  for (;;);
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via embOSView or high 
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time = OS_Time+1;
  unsigned int t_cnt = TMR0;
  if (TMCSR0_UF) {   /* If timer interrupt pending, */
    t_cnt = TMR0;    /* correct result              */
    time++;
  }
  return (OS_PCLK_TIMER/1000)*time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*/
__interrupt void OS_ISR_Tick (void) {
  OS_CallNestableISR(_ISR_Tick);
}

#if OS_UART_USED
/*********************************************************************
*
*       Communication for embOSView
*
**********************************************************************
*/

#define BRG_DIV ((OS_PCLK_UART / OS_BAUDRATE) - 1)
#if (BRG_DIV > 0xFFFFuL)
  #error "Baudrate for UART can not be generated"
#endif

#if   (OS_UART == 0)
  #define _OS_UART_RX_INT_INDEX 79
  #define _OS_UART_TX_INT_INDEX 80
  #define _USART_SIN_BIT 2  
  #define _BGRx          BGR0
  #define _SSRx          SSR0
  #define _SMRx          SMR0
  #define _SCRx          SCR0
  #define _TDRx          TDR0
  #define _RDRx          RDR0
  #define _SSRx_RIE      SSR0_RIE
  #define _SSRx_TIE      SSR0_TIE
  #define _SCRx_CRE      SCR0_CRE
#elif (OS_UART == 1)
  #define _OS_UART_RX_INT_INDEX 81
  #define _OS_UART_TX_INT_INDEX 82
  #define _USART_SIN_BIT 5  
  #define _BGRx          BGR1
  #define _SSRx          SSR1
  #define _SMRx          SMR1
  #define _SCRx          SCR1
  #define _TDRx          TDR1
  #define _RDRx          RDR1
  #define _SSRx_RIE      SSR1_RIE
  #define _SSRx_TIE      SSR1_TIE
  #define _SCRx_CRE      SCR1_CRE
#else
  #error "Selected UART not supported."
#endif

/*********************************************************************
*
*       _ISR_Tx()
*       Tx interrupt handler for UART1
*/
static void _ISR_Tx (void) {
  if (OS_OnTx()) {      /*  No more characters to send ? */
    _SSRx_TIE = 0;      /*  => disable Tx interrupt      */
  }
}

/*********************************************************************
*
*       _ISR_Rx()
*       Rx interrupt handler for UART1
*/
static void _ISR_Rx (void) {
  int Data;

  Data = _RDRx;
  if (_SSRx & 0xE0) {   /* Check if errors occurred */
    _SCRx_CRE = 0;      /* Reset error              */
  } else {
    OS_OnRx(Data);
  }
}

/*********************************************************************
*
*       OS_COM_Init
*/
void OS_COM_Init(void) {
  OS_IncDI();
  //
  // Setup interrupt controller
  //
  ICR = (_OS_UART_RX_INT_INDEX << 8) | _OS_UART_RX_INT_PRIORITY;
  ICR = (_OS_UART_TX_INT_INDEX << 8) | _OS_UART_TX_INT_PRIORITY;
  //
  // Initialize hardware for USART
  //
  DDR08    &= ~(1 << _USART_SIN_BIT);  // set SIN1 as input
  PIER08   |=  (1 << _USART_SIN_BIT);  // enable port input

  _BGRx     = BRG_DIV;    // Set calculated value for baud rate generator
  _SSRx     = 0;          // LSB first, Disable Rx/Tx interrupts
  _SMRx     = 0x0d;       // enable SOT1, Reset, asynchronous, normal mode
  _SCRx     = 0x17;       // 8N1, clear errors, enable Rx, Tx

  _SSRx_RIE = 1;          // Enable Rx interrupts
  OS_DecRI();
}

/*********************************************************************
*
*       OS_COM_Send1() for UART1:
*       Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(unsigned char c) {
  _TDRx = c;            /* Write character     */
  _SSRx_TIE = 1;        /* Enable Tx interrupt */
}

/*********************************************************************
*
*       OS_COM_ISR_Tx()
*       Tx interrupt handler for UART1
*/
__interrupt void OS_COM_ISR_Tx (void) {
  OS_CallNestableISR(_ISR_Tx);
}

/*********************************************************************
*
*       OS_COM_ISR_Rx()
*       Rx interrupt handler for UART1
*/
__interrupt void OS_COM_ISR_Rx (void) {
  OS_CallNestableISR(_ISR_Rx);
}

#else  /* UART handlers not defined, using dummies */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);           /* Avoid compiler warning                   */
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif /* OS_UART_USED */

/****** End Of File *************************************************/
